Attribute VB_Name = "EZ_USB"
Private Type ANCHOR_DOWNLOAD_CONTROL
  Offset As Integer
End Type

Private Type BULK_TRANSFER_CONTROL
  PipeNum As Long
End Type

Private Type VENDOR_REQUEST_IN
  bRequest As Byte
  lValue As Integer
  lIndex As Integer
  lLength As Integer
  bDirection As Byte
  bData As Byte
End Type

Private Type SET_INTERFACE_IN
  interfaceNum As Byte
  alternateSetting As Byte
End Type


Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, _
                                  ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, _
                                  lpSecurityAttributes As Long, _
                                  ByVal dwCreationDisposition As Long, _
                                  ByVal dwFlagsAndAttributes As Long, _
                                  ByVal hTemplateFile As Long) As Long
Declare Function DeviceIoControl Lib "kernel32" (ByVal hDevice As Long, ByVal dwIoControlCode As Long, _
                                  lpInBuffer As Any, ByVal nInBufferSize As Long, lpOutBuffer As Any, _
                                  ByVal nOutBufferSize As Long, lpBytesReturned As Long, _
                                  lpOverlapped As Long) As Long
Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

' Konstanten fr Function 'CreateFile'
Const GENERIC_READ = &H80000000
Const GENERIC_WRITE = &H40000000
Const FILE_SHARE_WRITE = &H2
Const FILE_SHARE_READ = &H1
Const OPEN_EXISTING = &H3

' Functionscode-Konstanten fr 'DeviceIoControl'
Const IOCTL_Ezusb_GET_PIPE_INFO& = &H222000
Const IOCTL_Ezusb_GET_DEVICE_DESCRIPTOR = &H222004
Const IOCTL_Ezusb_GET_CONFIGURATION_DESCRIPTOR = &H222008
Const IOCTL_Ezusb_BULK_OR_INTERRUPT_WRITE = &H22200C
Const IOCTL_Ezusb_BULK_OR_INTERRUPT_READ = &H222010
Const IOCTL_Ezusb_VENDOR_REQUEST = &H222014
Const IOCTL_Ezusb_GET_CURRENT_CONFIG = &H222018
Const IOCTL_Ezusb_ANCHOR_DOWNLOAD = &H22201C
Const IOCTL_Ezusb_ISO = &H222028
Const IOCTL_Ezusb_RESET = &H222031
Const IOCTL_Ezusb_RESETPIPE = &H222035
Const IOCTL_Ezusb_ABORTPIPE = &H22203D
Const IOCTL_Ezusb_SETINTERFACE = &H222040
Const IOCTL_Ezusb_GET_STRING_DESCRIPTOR = &H222044
Const IOCTL_EZUSB_BULK_READ = &H22204E
Const IOCTL_EZUSB_BULK_WRITE = &H222051
Const IOCTL_EZUSB_GET_CURRENT_FRAME_NUMBER = &H222054
Const IOCTL_EZUSB_VENDOR_OR_CLASS_REQUEST = &H222059
Const IOCTL_EZUSB_GET_LAST_ERROR = &H22205C
Const IOCTL_EZUSB_ISO_LOOPBACK = &H222061
Const IOCTL_EZUSB_ISO_READ = &H222066
Const IOCTL_EZUSB_ISO_WRITE = &H222069
Const IOCTL_EZUSB_ANCHOR_DOWNLOAD_2 = &H22206D
Const IOCTL_EZUSB_BULK_LATENCY_TEST = &H222070
Const IOCTL_EZUSB_GET_DRIVER_VERSION = &H222074

' Konstanten fr Transfer-Richtung in 'DeviceIoControl'
Const DirectionWrite = 0
Const DirectionRead = 1

' Fehlerkonstanten
Public Const Err_No% = 0                 ' kein Fehler aufgetreten
Public Const Err_InvalideHandle% = 1     ' kein gltiges Device-Handle
Public Const Err_CloseDriver% = 2        ' Treiber konnte nicht geschlossen werden
Public Const Err_InvalidFileName% = 4    ' Datei existiert nicht
Public Const Err_DeviceIOCtl% = 8        ' Funktion DeviceI0Control hat 'False' zurckgeliefert
Public Const Err_Array% = 16             ' Bytearray ist zu klein fr zu bertragende Byte-Zahl
Public Const Err_ValueCnt% = 32          ' zu bertragende Byte-Zahl ist zu gro oder zu klein


Private Sub CloseDriver(ByVal DeviceHandle As Long, Err%)
  Dim Dummy As Long
  ' Treiber schlieen
  Dummy = CloseHandle(DeviceHandle)
  If Dummy = 0 Then                   ' Funktion war nicht erfolgreich
    Err = Err Or Err_CloseDriver
  End If
End Sub

Public Sub DownloadBin(FileName As String, Err As Integer)
  Dim ByteArray(8191) As Byte
  Dim ArraySize As Long
  Dim nBytes As Long
  Dim DeviceHandle As Long
  Dim Dummy As Long
  Dim I As Integer
  Dim BinFile As Integer
  Dim DownLoadControl As ANCHOR_DOWNLOAD_CONTROL
  Dim bresult As Boolean
  
  '*** einlesen des Binrfiles in ByteArray ***
  I = 0
  BinFile = FreeFile
  On Error GoTo ErrHandle
  Open FileName For Binary As BinFile
  While Not EOF(BinFile)
    Get BinFile, , ByteArray(I)
    I = I + 1
  Wend
  Close BinFile
  ' Ungltiger Dateiname / Wenn Datei nicht vorhanden ist
  ' wird sie angelegt und ist leer ( i% = 1 )
  If I% = 1 Then
    Err = Err Or Err_InvalidFileName
    Exit Sub
  ElseIf I% > 8191 Then                 ' Datei zu gro
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  ArraySize = I
  ' Anzeigen der Arraywerte
  For I = 0 To ArraySize
    'MsgBox "ByteArray(" + Str(I) + ") =" + Str(ByteArray(I)), vbOKOnly
  Next I%
  DownLoadControl.Offset = 0            ' ab Adresse 0 ablegen
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  ' Block laden
  bresult = DeviceIoControl(DeviceHandle, IOCTL_EZUSB_ANCHOR_DOWNLOAD_2, _
                            DownLoadControl, 2, ByteArray(0), ArraySize, _
                            nBytes, gOverlaped)
  If bresult = False Then           ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
  Exit Sub
ErrHandle:
  Err = Err Or Err_InvalidFileName
End Sub

Public Sub DownloadHex(FileName As String, Err As Integer)
  Dim ByteArray(8191) As Byte: Dim ArraySize&
  Dim BytesPerLine As Double
  Dim nBytes&:          Dim DeviceHandle&
  Dim Dummy&:           Dim I%
  Dim HexFile%:      Dim Zeile$
  Dim DownLoadControl As ANCHOR_DOWNLOAD_CONTROL
  Dim Adr%: Dim MaxAdr%
  
  '*** Einlesen des Hexfiles in ByteArray ***
  HexFile = FreeFile
  On Error GoTo ErrHandle
  Open FileName For Input As HexFile
  While Not EOF(HexFile)
    Line Input #HexFile, Zeile
    BytesPerLine = Val("&H" + Mid$(Zeile, 2, 2)) ' Anzahl der Zeichen pro Zeile
    If BytesPerLine > 0 Then                     ' Anzahl grer 0
      Adr% = Val("&H" + Mid$(Zeile, 4, 4))       ' Adresse des ersten Bytes der Zeile
      For I% = 1 To BytesPerLine
        ByteArray(Adr) = Val("&H" + Mid$(Zeile, 8 + I * 2, 2))
        ArraySize = Adr
        Adr = Adr + 1
      Next I%
    End If
  Wend
  Close HexFile
  ' Laden des Byte-Arrays
  If ArraySize > 8191 Then              ' Datei zu gro
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  DownLoadControl.Offset = 0            ' ab Adresse 0 ablegen
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  ' Block laden
  bresult = DeviceIoControl(DeviceHandle, IOCTL_EZUSB_ANCHOR_DOWNLOAD_2, _
                            DownLoadControl, 2, ByteArray(0), ArraySize, _
                            nBytes, gOverlaped)
  If bresult = False Then           ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
  Exit Sub
ErrHandle:
  Err = Err Or Err_InvalidFileName

End Sub

Private Function OpenDriver(Err%) As Long
  OpenDriver = CreateFile("\\.\ezusb-0", GENERIC_WRITE, _
                            FILE_SHARE_WRITE, ByVal 0&, _
                            OPEN_EXISTING, 0, 0)
  If OpenDriver < 0 Then                ' Treiber konnte nicht geffnet werden
    Err% = Err% Or Err_InvalideHandle
  End If
End Function

Sub ProgReset(Err%)
  Dim DeviceHandle As Long
  Dim Dummy As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim MyRequest As VENDOR_REQUEST_IN
  
  '**** Variablen initialisieren ****
  MyRequest.bRequest = &HA0
  MyRequest.lValue = &H7F92
  MyRequest.lIndex = 0
  MyRequest.lLength = 1
  MyRequest.bDirection = DirectionWrite
  MyRequest.bData = 1 ' Reset
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  ' Rcksetzen des Controllers
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_VENDOR_REQUEST, _
                            MyRequest, 10, ByVal 0&, 0, nBytes, 0)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub

Sub ProgStart(Err%)
  Dim DeviceHandle As Long
  Dim Dummy As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim MyRequest As VENDOR_REQUEST_IN
  
  '**** Variablen initialisieren ****
  MyRequest.bRequest = &HA0
  MyRequest.lValue = &H7F92
  MyRequest.lIndex = 0
  MyRequest.lLength = 1
  MyRequest.bDirection = DirectionWrite
  MyRequest.bData = 0 ' Reset = 0
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  ' IO-Transfer
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_VENDOR_REQUEST, _
                            MyRequest, 10, &H0, 0, nBytes, gOverlaped)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub



Public Function RdRAM(ByVal Adr As Long, Err%) As Byte
  Dim DeviceHandle As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim MyRequest As VENDOR_REQUEST_IN
  Dim Buffer(1) As Byte
  Dim Dummy As Long
  
  MyRequest.bRequest = &HA0
  MyRequest.lValue = Adr
  MyRequest.lIndex = 0
  MyRequest.lLength = 1
  MyRequest.bDirection = DirectionRead
  MyRequest.bData = 0
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  ' Byte schreiben
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_VENDOR_REQUEST, _
                            MyRequest, 10, Buffer(0), 2, nBytes, 0)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
  RdRAM = Buffer(0)
End Function
Public Sub ReadBulkData(ByVal PipeNumber As Long, ByVal NumberOfBytes As Long, ByteBlock() As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresultn As Boolean
  Dim nBytes As Long
  Dim BulkControl As BULK_TRANSFER_CONTROL
  Dim BufferSize As Long
  Dim MinIndex As Long
  '*********************************
  BulkControl.PipeNum = PipeNumber
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  MinIndex = LBound(ByteBlock)                               ' kleinster Index
  BufferSize = UBound(ByteBlock) - MinIndex + 1              ' grter Index
  If (NumberOfBytes > 1024) Or (NumberOfBytes < 1) Then      ' Wertezahl berprfen
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  If BufferSize < NumberOfBytes Then                         ' Array zu klein
    Err = Err Or Err_Array
    Exit Sub
  End If
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  bresult = DeviceIoControl(DeviceHandle, IOCTL_EZUSB_BULK_READ, BulkControl, _
                            Len(BulkControl), ByteBlock(MinIndex), NumberOfBytes, _
                            nBytes, ByVal 0&)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub

Public Sub ReadRAMbytes(ByVal Adr As Long, ByVal NumberOfBytes As Long, ByteBlock() As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim MyRequest As VENDOR_REQUEST_IN
  Dim BufferSize As Long
  Dim MinIndex As Long
  
  MyRequest.bRequest = &HA0
  MyRequest.lValue = Adr
  MyRequest.lIndex = 0
  MyRequest.lLength = NumberOfBytes                          ' maximal 1024 Byte
  MyRequest.bDirection = DirectionRead                       ' lesen
  MyRequest.bData = 0
  
  MinIndex = LBound(ByteBlock)                               ' kleinster Index
  BufferSize = UBound(ByteBlock) - MinIndex + 1              ' grter Index
  If (NumberOfBytes > 1024) Or (NumberOfBytes < 2) Then      ' Wertezahl berprfen
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  If BufferSize < NumberOfBytes Then                         ' Array zu klein
    Err = Err Or Err_Array
    Exit Sub
  End If
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  'Byte schreiben
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_VENDOR_REQUEST, _
                            MyRequest, 10, ByteBlock(MinIndex), BufferSize, _
                            nBytes, 0)
  If bresult = False Then             ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub

Public Sub SetInterface(ByVal InterfaceNumber As Byte, ByVal Setting As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim SetInterface As SET_INTERFACE_IN
  '*************************
  SetInterface.interfaceNum = InterfaceNumber
  SetInterface.alternateSetting = Setting
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_SETINTERFACE, _
                            SetInterface, Len(SetInterface), ByVal 0&, _
                            0, nBytes, ByVal 0&)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub

Public Sub WriteBulkData(ByVal PipeNumber As Long, ByVal NumberOfBytes As Long, ByteBlock() As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresultn As Boolean
  Dim nBytes As Long
  Dim BulkControl As BULK_TRANSFER_CONTROL
  Dim BufferSize As Long
  Dim MinIndex As Long
  '*********************************
  BulkControl.PipeNum = PipeNumber
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  MinIndex = LBound(ByteBlock)                               ' kleinster Index
  BufferSize = UBound(ByteBlock) - MinIndex + 1              ' grter Index
  If (NumberOfBytes > 1024) Or (NumberOfBytes < 1) Then      ' Wertezahl berprfen
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  If BufferSize < NumberOfBytes Then                         ' Array zu klein
    Err = Err Or Err_Array
    Exit Sub
  End If
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  bresult = DeviceIoControl(DeviceHandle, IOCTL_EZUSB_BULK_WRITE, BulkControl, _
                            Len(BulkControl), ByteBlock(MinIndex), NumberOfBytes, _
                            nBytes, ByVal 0&)
  If bresult = False Then         ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub

Public Sub WriteRAMbytes(ByVal Adr As Long, ByVal NumberOfBytes As Long, ByteBlock() As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim DownLoadControl As ANCHOR_DOWNLOAD_CONTROL
  Dim MinIndex As Long
  Dim BufferSize As Long
  '*****************************************************************************************
  MinIndex = LBound(ByteBlock)                              ' kleinster Index
  BufferSize = UBound(ByteBlock) - MinIndex + 1             ' grter Index
  If (NumberOfBytes > 8191) Or (NumberOfBytes < 2) Then     ' Wertezahl berprfen
    Err = Err Or Err_ValueCnt
    Exit Sub
  End If
  If BufferSize < NumberOfBytes Then                        ' Array zu klein
    Err = Err Or Err_Array
    Exit Sub
  End If
  DownLoadControl.Offset = Adr                              ' schreiben ab Adresse ...
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  'Byte schreiben
  bresult = DeviceIoControl(DeviceHandle, IOCTL_EZUSB_ANCHOR_DOWNLOAD_2, _
                            DownLoadControl, 2, ByteBlock(0), NumberOfBytes, _
                            nBytes, 0&)
  If bresult = False Then             ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub
Public Sub WrRAM(ByVal Adr As Long, ByVal WrValue As Byte, Err%)
  Dim DeviceHandle As Long
  Dim bresult As Boolean
  Dim nBytes As Long
  Dim MyRequest As VENDOR_REQUEST_IN
  
  MyRequest.bRequest = &HA0
  MyRequest.lValue = Adr
  MyRequest.lIndex = 0
  MyRequest.lLength = 1
  MyRequest.bDirection = DirectionWrite
  MyRequest.bData = WrValue
  ' Treiber ffnen
  DeviceHandle = OpenDriver(Err%)
  'Byte schreiben
  bresult = DeviceIoControl(DeviceHandle, IOCTL_Ezusb_VENDOR_REQUEST, _
                            MyRequest, 10, ByVal 0&, 0, nBytes, 0)
  If bresult = False Then             ' Funktion war nicht erfolgreich
    Err = Err Or Err_DeviceIOCtl
  End If
  ' Treiber schlieen
  Call CloseDriver(DeviceHandle, Err%)
End Sub
